<?php
function imagefield_zip_js($type_name, $field_name) {  
  $result = false;

  $validators = array(
    'file_validate_extensions' => array('zip'),
    'imagefield_zip_is_valid_zip' => array(),
  );
  
  $images = array();
  if ($file = file_save_upload($field_name .'_zip', $validators)) {
    $extracted = imagefield_zip_extract($file->filepath);
    $images = array_filter($extracted, '_imagefield_zip_is_archived_image');
  }
  
  // Delete the zip file as we extracted and used everything we wanted from it.
  imagefield_zip_delete_file($file);
  
  $result = _imagefield_zip_js($type_name, $field_name, $images);
  
  $data = theme('status_messages') . $result;
  if ($result !== false) {
    $return = array('data' => $data, 'status' => true);
  }
  else {
    $return = array('data' => $data);
  }
  
  print drupal_to_js($return);
  exit;
}

function _imagefield_zip_js($type_name, $field_name, $files) {
  $field = content_fields($field_name, $type_name);
  
  if (empty($field) || empty($_POST['form_build_id'])) {
    // Invalid request.
    return false;
  }
  
  // Build the new form.
  $form_state = array('submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);

  if (!$form) {
    // Invalid form_build_id.
    return false;
  }

  // form_get_cache() doesn't yield the original $form_state,
  // but form_builder() does. Needed for retrieving the file array.
  $built_form = $form;
  $built_form_state = $form_state;

  $built_form += array('#post' => $_POST);
  $built_form = form_builder($_POST['form_id'], $built_form, $built_form_state);

  // Clean ids, so that the same element doesn't get a different element id
  // when rendered once more further down.
  form_clean_id(NULL, TRUE);

  // Ask CCK for the replacement form element. Going through CCK gets us
  // the benefit of nice stuff like '#required' merged in correctly.
  module_load_include('inc', 'content', 'includes/content.node_form');
  $field_element = content_field_form($form, $built_form_state, $field);
  $dest = filefield_widget_file_path($field_element);
  $widget_function = $field['widget']['module'] .'_widget';
  
  // Go through each file and create and append it to an imagefield
  for ($delta = 0, $file = reset($files); $file; ++$delta) {
    if (!isset($field_element[$field_name][$delta]) || empty($field_element[$field_name][$delta]['#default_value'])) {
      $delta_element = $widget_function($built_form, $built_form_state, $field, array(), $delta);
      
      if ($file = _imagefield_zip_save_file($file->filepath, $delta_element['#upload_validators'], $dest)) {
        // Set the default value of the widget to the extracted file.
        $delta_element['#default_value'] = filefield_widget_value($delta_element, (array) $file);
        // We add a div around the new content to tell AHAH to let this fade in.
        $delta_element['#prefix'] = '<div class="ahah-new-content">';
        $delta_element['#suffix'] = '</div>';
        
        $field_element[$field_name][$delta] = $delta_element;
      }
      // If the file isn't valid then don't include the widget into the form but reuse the delta index.
      // Don't worry, we're still counting the file index so there won't be an infinate loop... I hope.
      else {
        --$delta;
      }
      $file = next($files);
    }
  }
  
  // Add the new element at the right place in the form.
  if (module_exists('fieldgroup') && ($group_name = _fieldgroup_field_get_group($type_name, $field_name))) {
    $form[$group_name][$field_name] = form_builder($_POST['form_id'], $field_element, $built_form_state);
  }
  else {
    $form[$field_name] = form_builder($_POST['form_id'], $field_element, $built_form_state);
  }

  // Write the (unbuilt, updated) form back to the form cache.
  form_set_cache($form_build_id, $form, $form_state);

  // Render the form for output.
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );
  drupal_alter('form', $form, array(), 'imagefield_zip_js');
  $form_state = array('submitted' => FALSE);
  $form = form_builder('imagefield_zip_js', $form, $built_form_state);
  $field_form = empty($group_name) ? $form[$field_name] : $form[$group_name][$field_name];

  $output = drupal_render($field_form);

  // AHAH is not being nice to us and doesn't know the "other" button (that is,
  // either "Upload" or "Delete") yet. Which in turn causes it not to attach
  // AHAH behaviours after replacing the element. So we need to tell it first.
  $javascript = drupal_add_js(NULL, NULL);
  if (isset($javascript['setting'])) {
    $output .= '<script type="text/javascript">jQuery.extend(Drupal.settings, '. drupal_to_js(call_user_func_array('array_merge_recursive', $javascript['setting'])) .');</script>';
  }

  // For some reason, file uploads don't like drupal_json() with its manual
  // setting of the text/javascript HTTP header. So use this one instead.
  $GLOBALS['devel_shutdown'] = false;
  return $output;
}

function _imagefield_zip_save_file($filepath, $validators = array(), $dest = false, $replace = FILE_EXISTS_RENAME) {
  global $user;
  
  // Add in our check of the the file name length.
  $validators['file_validate_name_length'] = array();
  
  $extensions = '';
  foreach ($user->roles as $rid => $name) {
    $extensions .= ' '. variable_get("upload_extensions_$rid",
    variable_get('upload_extensions_default', 'jpg jpeg gif png txt html doc xls pdf ppt pps odt ods odp'));
  }
  
  // Begin building file object.
  $file = new stdClass();
  $file->filename = file_munge_filename(trim(basename($filepath), '.'), $extensions);
  $file->filepath = $filepath;
  $file->filemime = file_get_mimetype($filepath);

  // Rename potentially executable files, to help prevent exploits.
  if (preg_match('/\.(php|pl|py|cgi|asp|js)$/i', $file->filename) && (substr($file->filename, -4) != '.txt')) {
    $file->filemime = 'text/plain';
    $file->filepath .= '.txt';
    $file->filename .= '.txt';
  }
  
  // If the destination is not provided, or is not writable, then use the
  // temporary directory.
  if (empty($dest) || file_check_path($dest) === FALSE) {
    $dest = file_directory_temp();
  }
  
  $file->destination = file_destination(file_create_path($dest .'/'. $file->filename), $replace);
  $file->filesize = filesize($filepath);

  // Call the validation functions.
  $errors = array();
  foreach ($validators as $function => $args) {
    array_unshift($args, $file);
    $errors = array_merge($errors, call_user_func_array($function, $args));
  }
  
  // Check for validation errors.
  if (!empty($errors)) {
    $message = t('The selected file %name could not be uploaded.', array('%name' => $file->filename));
    if (count($errors) > 1) {
      $message .= '<ul><li>'. implode('</li><li>', $errors) .'</li></ul>';
    }
    else {
      $message .= ' '. array_pop($errors);
    }
    
    form_set_error($source, $message);
    return 0;
  }
  
  $file->filepath = $file->destination;
  if (!file_move($filepath, $file->filepath)) {
    form_set_error($source, t('File upload error. Could not move uploaded file.'));
    watchdog('file', 'Upload error. Could not move uploaded file %file to destination %destination.', array('%file' => $file->filename, '%destination' => $file->filepath));
    return 0;
  }

  // If we made it this far it's safe to record this file in the database.
  $file->uid = $user->uid;
  $file->status = FILE_STATUS_TEMPORARY;
  $file->timestamp = time();
  drupal_write_record('files', $file);
  
  // Let modules add additional properties to the yet barebone file object.
  foreach (module_implements('file_insert') as $module) {
    $function =  $module .'_file_insert';
    $function($file);
  }
  
  return $file;
}